<?php
/**
 * app/plugins/notes/forms.php
 *
 * This houses all of the form checking functions for this plugin.
 *
 * @package TP Notes
 * @version 3.0
 * @author  Joey Kimsey <Joey@thetempusproject.com>
 * @link    https://TheTempusProject.com
 * @license https://opensource.org/licenses/MIT [MIT LICENSE]
 */
namespace TheTempusProject\Plugins\Notes;

use TheTempusProject\Bedrock\Functions\Input;
use TheTempusProject\Bedrock\Functions\Check;
use TheTempusProject\Classes\Forms;

class ContactsForms extends Forms {
    /**
     * Adds these functions to the form list.
     */
    public function __construct() {
        self::addHandler( 'createContact', __CLASS__, 'createContact' );
        self::addHandler( 'createPhonebook', __CLASS__, 'createPhonebook' );
        self::addHandler( 'editContact', __CLASS__, 'editContact' );
        self::addHandler( 'editPhonebook', __CLASS__, 'editPhonebook' );
    }

    public static function createContact() {
        if ( ! Input::exists( 'submit' ) ) {
            return false;
        }
        if ( ! Input::exists( 'first_name' ) && ! Input::exists( 'last_name' ) && ! Input::exists( 'nickname' ) ) {
            Check::addUserError( 'You must include a first, last, or nick-name.' );
            return false;
        }
        // if ( !self::token() ) {
        //     Check::addUserError( 'token - comment out later.' );
        //     return false;
        // }
        return true;
    }

    public static function createPhonebook() {
        if ( ! Input::exists( 'submit' ) ) {
            return false;
        }
        if ( ! Input::exists( 'title' ) ) {
            Check::addUserError( 'You must include a title.' );
            return false;
        }
        if ( ! Input::exists( 'color' ) ) {
            Check::addUserError( 'You must include a color.' );
            return false;
        }
        // if ( ! self::token() ) {
        //     Check::addUserError( 'token - comment out later.' );
        //     return false;
        // }
        return true;
    }

    public static function editContact() {
        if ( ! Input::exists( 'submit' ) ) {
            return false;
        }
        if ( ! Input::exists( 'first_name' ) && ! Input::exists( 'last_name' ) && ! Input::exists( 'nickname' ) ) {
            Check::addUserError( 'You must include a first, last, or nick-name.' );
            return false;
        }
        // if ( !self::token() ) {
        //     Check::addUserError( 'token - comment out later.' );
        //     return false;
        // }
        return true;
    }

    public static function editPhonebook() {
        if ( ! Input::exists( 'submit' ) ) {
            return false;
        }
        if ( ! Input::exists( 'title' ) ) {
            Check::addUserError( 'You must include a title.' );
            return false;
        }
        if ( ! Input::exists( 'color' ) ) {
            Check::addUserError( 'You must include a color.' );
            return false;
        }
        // if ( !self::token() ) {
        //     Check::addUserError( 'token - comment out later.' );
        //     return false;
        // }
        return true;
    }
}

new ContactsForms;