<?php
/**
 * app/plugins/dnd/controllers/admin/classes.php
 *
 * This is the classes admin controller.
 *
 * @package TTE Dungeons & Dragons
 * @version 3.0
 * @author  Joey Kimsey <Joey@tabletopelite.com>
 * @link    https://TableTopElite.com
 */
namespace TheTempusProject\Controllers\Admin;

use TheTempusProject\Houdini\Classes\Issues;
use TheTempusProject\Houdini\Classes\Views;
use TheTempusProject\Houdini\Classes\Components;
use TheTempusProject\Bedrock\Functions\Input;
use TheTempusProject\Bedrock\Functions\Check;
use TheTempusProject\Classes\Forms;
use TheTempusProject\Classes\AdminController;
use TheTempusProject\Models\Classes as ClassesModel;
use TheTempusProject\Houdini\Classes\Forms as HoudiniForms;

class Classes extends AdminController {
    protected static $classes;

    public function __construct() {
        parent::__construct();
        self::$title = 'Admin - D&D Classes';
        self::$classes = new ClassesModel;
    }

    public function index( $data = null ) {
        Views::view( 'dnd.admin.classes.list', self::$classes->list() );
    }

    public function view( $id = null ) {
        $data = self::$classes->findById( $id );
        if ( $data == false ) {
            Issues::add( 'error', 'Class not found.' );
            return $this->index();
        }
        Views::view( 'dnd.admin.classes.view', $data );
    }

    public function create( $data = null ) {
        Components::set( 'privacyDropdown', Views::simpleView('dnd.forms.privacyDropdown') );
        Components::set( 'versionDropdown', Views::simpleView('dnd.forms.versionDropdown') );
        $sourcebookSelect = HoudiniForms::getFormFieldHtml( 'sourcebookID', 'SourceBook', 'select', 'none', self::$sourcebooks->simpleList() );
        Components::set( 'sourcebookSelect', $sourcebookSelect );
        if ( !Input::exists( 'submit' ) ) {
            Views::view( 'dnd.admin.classes.create' );
        }
        if ( !Forms::check( 'createClass' ) ) {
            Issues::add( 'error', [ 'There was an error with your request.' => Check::userErrors() ] );
            return Views::view( 'dnd.admin.classes.create' );
        }
        if ( self::$classes->create( Input::post( 'name' )) ) {
            Issues::add( 'success', 'Class created' );
            return $this->index();
        } else {
            Issues::add( 'error', 'There was an error creating your class.' );
            return Views::view( 'dnd.admin.classes.create' );
        }
    }

    public function edit( $id = null ) {
        $data = self::$classes->findById( $id );
        if ( false == $data ) {
            return $this->index();
        }
        Components::set( 'privacyDropdown', Views::simpleView('dnd.forms.privacyDropdown', $data) );
        Components::set( 'versionDropdown', Views::simpleView('dnd.forms.versionDropdown', $data) );
        $sourcebookSelect = HoudiniForms::getFormFieldHtml( 'sourcebookID', 'SourceBook', 'select', $data->sourcebookID, self::$sourcebooks->simpleList() );
        Components::set( 'sourcebookSelect', $sourcebookSelect );
        if ( !Input::exists( 'submit' ) ) {
            return Views::view( 'dnd.admin.classes.edit', $data );
        }
        if ( !Forms::check( 'editClass' ) ) {
            Issues::add( 'error', [ 'There was an error with your request.' => Check::userErrors() ] );
            return Views::view( 'dnd.admin.classes.edit', $data );
        }
        if ( self::$classes->update( $id, Input::post( 'title' ), Input::post( 'suggestion' ), Input::post( 'approved' ) ) ) {
            Issues::add( 'success', 'Class updated' );
        } else {
            return Views::view( 'dnd.admin.classes.edit', $data );
        }
    }

    public function delete( $data = null ) {
        if ( Input::exists( 'submit' ) ) {
            $data = Input::post( 'CL_' );
        }
        if ( !self::$classes->delete( $data ) ) {
            Issues::add( 'error', 'There was an error with your request.' );
        } else {
            Issues::add( 'success', 'Class has been deleted' );
        }
        $this->index();
    }
}
