<?php
/**
 * app/plugins/dnd/models/gods.php
 *
 * This class is used for the manipulation of the dnd_gods database table.
 *
 * @package TTE Dungeons & Dragons
 * @version 3.0
 * @author  Joey Kimsey <Joey@tabletopelite.com>
 * @link    https://TableTopElite.com
 */
namespace TheTempusProject\Models;

use TheTempusProject\Canary\Canary as Debug;
use TheTempusProject\Classes\DatabaseModel;
use TheTempusProject\TheTempusProject as App;
use TheTempusProject\Bedrock\Classes\CustomException;
use TheTempusProject\Models\Sourcebooks as SourcebooksModel;

class Gods extends DatabaseModel {
    protected static $sourcebooks;
    public $tableName = 'dnd_gods';
    public $databaseMatrix = [
        // Name
        [ 'name', 'varchar', '32' ],
        [ 'pantheon', 'varchar', '32' ],
        [ 'domains', 'varchar', '32' ],
        [ 'symbol', 'varchar', '32' ],
        [ 'morality', 'varchar', '16' ], // good, neutral, evil
        [ 'order', 'varchar', '16' ], // lawful, neutral, chaotic
        
        // General
        [ 'createdBy', 'int', '11' ],
        [ 'createdAt', 'int', '11' ],
        [ 'privacy', 'varchar', '16' ],
        [ 'version', 'varchar', '5' ],
        [ 'sourcebookID', 'int', '11' ],
        [ 'shortDescription', 'text', '' ],
        [ 'description', 'text', '' ],
        [ 'image', 'text', '' ],
    ];

    /**
     * The model constructor.
     */
    public function __construct() {
        parent::__construct();
        self::$sourcebooks = new SourcebooksModel;
    }

    public function create( $name, $privacy, $version, $sourcebookID, $description = '', $shortDescription = '' ) {
        $fields = [
            'name' => $name,
            'privacy' => $privacy,
            'version' => $version,
            'sourcebookID' => $sourcebookID,
            'description' => $description,
            'shortDescription' => $shortDescription,
            'createdBy' => App::$activeUser->ID,
            'createdAt' => time(),
        ];
        if ( ! self::$db->insert( $this->tableName, $fields ) ) {
            new CustomException( 'Create' );
            Debug::error( "God: not created " . var_export($fields,true) );
            return false;
        }
        return self::$db->lastId();
    }

    public function update( $id, $name, $privacy, $version, $sourcebookID, $description = '', $shortDescription = '' ) {
        $fields = [
            'name' => $name,
            'privacy' => $privacy,
            'version' => $version,
            'sourcebookID' => $sourcebookID,
            'description' => $description,
            'shortDescription' => $shortDescription,
        ];
        if ( !self::$db->update( $this->tableName, $id, $fields ) ) {
            new CustomException( 'Update' );
            Debug::error( "God: $id not updated: $fields" );
            return false;
        }
        return true;
    }

    public function byUser( $limit = null ) {
        $whereClause = ['createdBy', '=', App::$activeUser->ID];
        if ( empty( $limit ) ) {
            $objects = self::$db->get( $this->tableName, $whereClause );
        } else {
            $objects = self::$db->get( $this->tableName, $whereClause, 'ID', 'DESC', [0, $limit] );
        }
        if ( !$objects->count() ) {
            Debug::info( 'No Gods found.' );
            return false;
        }
        return $this->filter( $objects->results() );
    }
}
